package com.tencent.liteav.demo.player;

import static android.view.View.SYSTEM_UI_FLAG_FULLSCREEN;
import static android.view.View.SYSTEM_UI_FLAG_HIDE_NAVIGATION;

import static androidx.core.view.ViewCompat.getWindowSystemUiVisibility;

import android.annotation.TargetApi;
import android.app.Activity;
import android.content.Context;
import android.content.res.Configuration;
import android.graphics.Insets;
import android.os.Build;
import android.view.DisplayCutout;
import android.view.Surface;
import android.view.WindowInsets;
import android.view.WindowManager;

import androidx.annotation.RequiresApi;


public class ViewInsets {

    public static class ViewportMetrics {
        public int viewPaddingTop = 0;
        public int viewPaddingRight = 0;
        public int viewPaddingBottom = 0;
        public int viewPaddingLeft = 0;
        public int viewInsetTop = 0;
        public int viewInsetRight = 0;
        public int viewInsetBottom = 0;
        public int viewInsetLeft = 0;
    }

    private enum ZeroSides {
        NONE,
        LEFT,
        RIGHT,
        BOTH
    }

    private static ZeroSides calculateShouldZeroSides(Activity activity) {
        // We get both orientation and rotation because rotation is all 4
        // rotations relative to default rotation while orientation is portrait
        // or landscape. By combining both, we can obtain a more precise measure
        // of the rotation.
        Context context = activity.getBaseContext();
        int orientation = context.getResources().getConfiguration().orientation;
        int rotation =
                ((WindowManager) context.getSystemService(Context.WINDOW_SERVICE))
                        .getDefaultDisplay()
                        .getRotation();

        if (orientation == Configuration.ORIENTATION_LANDSCAPE) {
            if (rotation == Surface.ROTATION_90) {
                return ZeroSides.RIGHT;
            } else if (rotation == Surface.ROTATION_270) {
                // In android API >= 23, the nav bar always appears on the "bottom" (USB) side.
                return Build.VERSION.SDK_INT >= 23 ? ZeroSides.LEFT : ZeroSides.RIGHT;
            }
            // Ambiguous orientation due to landscape left/right default. Zero both sides.
            else if (rotation == Surface.ROTATION_0 || rotation == Surface.ROTATION_180) {
                return ZeroSides.BOTH;
            }
        }
        // Square orientation deprecated in API 16, we will not check for it and return false
        // to be safe and not remove any unique padding for the devices that do use it.
        return ZeroSides.NONE;
    }


    public static ViewportMetrics applyWindowInsets(Activity activity, WindowInsets insets) {
        ViewportMetrics viewportMetrics = new ViewportMetrics();

        boolean statusBarVisible = (SYSTEM_UI_FLAG_FULLSCREEN & getWindowSystemUiVisibility(activity.getWindow().getDecorView())) == 0;
        boolean navigationBarVisible =
                (SYSTEM_UI_FLAG_HIDE_NAVIGATION & getWindowSystemUiVisibility(activity.getWindow().getDecorView())) == 0;

        if (Build.VERSION.SDK_INT >= Build.VERSION_CODES.R) {
            int mask = 0;
            if (navigationBarVisible) {
                mask = mask | android.view.WindowInsets.Type.navigationBars();
            }
            if (statusBarVisible) {
                mask = mask | android.view.WindowInsets.Type.statusBars();
            }
            Insets uiInsets = insets.getInsets(mask);
            viewportMetrics.viewPaddingTop = uiInsets.top;
            viewportMetrics.viewPaddingRight = uiInsets.right;
            viewportMetrics.viewPaddingBottom = uiInsets.bottom;
            viewportMetrics.viewPaddingLeft = uiInsets.left;

            Insets imeInsets = insets.getInsets(android.view.WindowInsets.Type.ime());
            viewportMetrics.viewInsetTop = imeInsets.top;
            viewportMetrics.viewInsetRight = imeInsets.right;
            viewportMetrics.viewInsetBottom = imeInsets.bottom;
            viewportMetrics.viewInsetLeft = imeInsets.left;
        } else {
            // We zero the left and/or right sides to prevent the padding the
            // navigation bar would have caused.
            ZeroSides zeroSides = ZeroSides.NONE;
            if (!navigationBarVisible) {
                zeroSides = calculateShouldZeroSides(activity);
            }

            // Status bar (top), navigation bar (bottom) and left/right system insets should
            // partially obscure the content (padding).
            viewportMetrics.viewPaddingTop = statusBarVisible ? insets.getSystemWindowInsetTop() : 0;
            viewportMetrics.viewPaddingRight =
                    zeroSides == ZeroSides.RIGHT || zeroSides == ZeroSides.BOTH
                            ? 0
                            : insets.getSystemWindowInsetRight();
            viewportMetrics.viewPaddingBottom =
                    navigationBarVisible && guessBottomKeyboardInset(activity, insets) == 0
                            ? insets.getSystemWindowInsetBottom()
                            : 0;
            viewportMetrics.viewPaddingLeft =
                    zeroSides == ZeroSides.LEFT || zeroSides == ZeroSides.BOTH
                            ? 0
                            : insets.getSystemWindowInsetLeft();

            // Bottom system inset (keyboard) should adjust scrollable bottom edge (inset).
            viewportMetrics.viewInsetTop = 0;
            viewportMetrics.viewInsetRight = 0;
            viewportMetrics.viewInsetBottom = guessBottomKeyboardInset(activity, insets);
            viewportMetrics.viewInsetLeft = 0;
        }

        if (Build.VERSION.SDK_INT >= Build.VERSION_CODES.Q) {
            DisplayCutout cutout = insets.getDisplayCutout();
            if (cutout != null) {
                Insets waterfallInsets = Insets.NONE;

                if (Build.VERSION.SDK_INT >= Build.VERSION_CODES.R) {
                    waterfallInsets = cutout.getWaterfallInsets();
                }

                viewportMetrics.viewPaddingTop =
                        Math.max(
                                Math.max(viewportMetrics.viewPaddingTop, waterfallInsets.top),
                                cutout.getSafeInsetTop());
                viewportMetrics.viewPaddingRight =
                        Math.max(
                                Math.max(viewportMetrics.viewPaddingRight, waterfallInsets.right),
                                cutout.getSafeInsetRight());
                viewportMetrics.viewPaddingBottom =
                        Math.max(
                                Math.max(viewportMetrics.viewPaddingBottom, waterfallInsets.bottom),
                                cutout.getSafeInsetBottom());
                viewportMetrics.viewPaddingLeft =
                        Math.max(
                                Math.max(viewportMetrics.viewPaddingLeft, waterfallInsets.left),
                                cutout.getSafeInsetLeft());
            }
        }

        return viewportMetrics;
    }

    @TargetApi(20)
    @RequiresApi(20)
    private static int guessBottomKeyboardInset(Activity activity, WindowInsets insets) {
        int screenHeight = activity.getWindow().getDecorView().getHeight();
        // Magic number due to this being a heuristic. This should be replaced, but we have not
        // found a clean way to do it yet (Sept. 2018)
        final double keyboardHeightRatioHeuristic = 0.18;
        if (insets.getSystemWindowInsetBottom() < screenHeight * keyboardHeightRatioHeuristic) {
            // Is not a keyboard, so return zero as inset.
            return 0;
        } else {
            // Is a keyboard, so return the full inset.
            return insets.getSystemWindowInsetBottom();
        }
    }

}
